;*******************************************************************************
;* Digitlis technika 11. laboratrium                                         *
;* Bit-bang SPI kommunikci a TMP121 hmrvel.                               *
;*******************************************************************************
DEF DIG0 0x90                ; Kijelz DIG0 adatregiszter          (rhat/olvashat)
DEF DIG1 0x91                ; Kijelz DIG1 adatregiszter          (rhat/olvashat)
DEF DIG2 0x92                ; Kijelz DIG2 adatregiszter          (rhat/olvashat)
DEF DIG3 0x93                ; Kijelz DIG3 adatregiszter          (rhat/olvashat)
DEF ADO  0xA0                ; GPIO A kimeneti adatregiszter       (rhat/olvashat)
DEF ADI  0xA1                ; GPIO A adat az I/O lbakon          (rhat/olvashat)
DEF ADR  0xA2                ; GPIO A irnyregister (0: be, 1: ki) (rhat/olvashat)

    CODE
    
;*******************************************************************************
;* A program kezdete. A programmemria 0x00 s a 0x01 cme a reset, illetve    *
;* a megszakts vektor. Ide ugr utastsokat kell elhelyezni, amelyek a      *
;* megfelel programrszre ugranak. Ha nem hasznlunk megszaktst, akkor a    *
;* program kezddhet a 0x00 cmen is.                                          *
;*******************************************************************************
start:

loop:

    jmp     loop            ;Ugrs a ciklus elejre.

    
;*******************************************************************************
;* A GPIO A port inicializlsa a TMP121 hmrvel trtn kommunikcihoz.    *
;* GPIO A[0]: CSn  -> kimenet, kezdetben 1                                     *
;* GPIO A[1]: SCK  -> kimenet, kezdetben 0                                     *
;* GPIO A[2]: MISO -> bemenet                                                  *
;*******************************************************************************
gpio_init:

    rts                     ;Visszatrs a hvhoz.


;*******************************************************************************
;* A 16 bites hmrsklet rtk beolvassa a TMP121 hmrrl.                 *
;* Visszatrsi rtk:                                                         *
;*   r6: A 16 bites hmrsklet rtk als 8 bitje (LSB).                      *
;*   r7: A 16 bites hmrsklet rtk fels 8 bitje (MSB).                     *
;* Munkaregiszternek, illetve ciklusvltoznak hasznljuk az r8-at s r9-et.   *
;*******************************************************************************
tmp121_rd:

    rts                     ; Visszatrs a hvhoz.
 

;*******************************************************************************
;* Balra igaztott trtrsz szorzsa szzzal a 2 digites megjelentshez.      *
;* Paramterek:                                                                *
;*   r8: A balra igaztott trtrsz (MSb az 1/2 helyirtk).                   *
;* Visszatrsi rtk:                                                         *
;*   r8: A szorzat.                                                            *
;*******************************************************************************
frac_mul100:

    rts                     ;Visszatrs a hvhoz.
    

;*******************************************************************************
;* Vrakozs 1 msodpercig szoftveres idztssel (a 10. laboron mr volt).    *
;* Hasznljuk az r15:r14:r13 regiszter hrmast a ciklusvltoznak.             *
;*******************************************************************************
wait_1s:

    rts


;*******************************************************************************
;* Kt digites binris -> BCD talakts (SHIFT & ADD3 algoritmus).            *
;* Paramterek:                                                                *
;*   r8: Az talaktand binris szm.                                         *
;* Visszatrsi rtk:                                                         *
;*   r8: A kapott BCD szm.                                                    *
;*******************************************************************************
bcd_conv:
    mov     r9, #0          ; Az r9 trolja az eredmnyt.
    mov     r10, #8         ; A htralv shiftelsek szma.
bcd_loop:
    mov     r11, r9         ; Az egyesek vizsglata: ha 5 vagy tbb, akkor
    and     r11, #0x0f      ; hrmat hozz kell adni.
    cmp     r11, #0x05
    jc      no_corr1
    add     r9, #0x03
no_corr1:
    mov     r11, r9         ; A tizesek vizsglata: ha 5 vagy tbb, akkor
    and     r11, #0xf0      ; hrmat hozz kell adni.
    cmp     r11, #0x50
    jc      no_corr10
    add     r9, #0x30
no_corr10:
    sl0     r8              ; Az r9:r8 regiszterprt balra shifteljk.
    rlc     r9
    sub     r10, #1         ; Cskkentjk a shiftelsek szmt.
    jnz     bcd_loop        ; Ugrs, ha nincs mg vge a konverzinak.
    mov     r8, r9
    rts                     ; Visszatrs a hvhoz.


;*******************************************************************************
;* Kt digites megjelents a htszegmenses kijelzn (a 9. laboron mr volt).  *
;* Paramterek:                                                                *
;*   r8 : A megjelentend szm.                                               *
;*   r9 : Az egyesek helyirtkhez tartoz digit regiszter cme.              *
;*   r10: Tizedespont vezrls (bit 1: tizesek, bit 0: egyesek).               *
;*******************************************************************************
disp_wr:
    mov     r11, r8             ;Msolat ksztse a szmrl.
    and     r11, #0x0f          ;Trljk a fels 4 bitet.
    add     r11, #bin_7seg      ;A htszegmenses dekder tblzat elem cme.
    mov     r11, (r11)          ;A tblzat elem beolvassa.
    tst     r10, #0x01          ;A tizedespont bekapcsolsnak vizsglata.
    jz      disp_wr_skip1       ;Ugrs, ha nem kell bekapcsolni.
    or      r11, #0x80          ;A tizedespont bekapcsolsa.
disp_wr_skip1:
    mov     (r9), r11           ;Megjelents a kijelzn.
    
    add     r9, #1              ;A mutatt a kvetkez digitre lltjuk.
    
    mov     r11, r8             ;Msolat ksztse a szmrl.
    swp     r11                 ;A megjelentend rtk az als 4 bitbe kerl.
    and     r11, #0x0f          ;Trljk a fels 4 bitet.
    add     r11, #bin_7seg      ;A htszegmenses dekder tblzat elem cme.
    mov     r11, (r11)          ;A tblzat elem beolvassa.
    tst     r10, #0x02          ;A tizedespont bekapcsolsnak vizsglata.
    jz      disp_wr_skip2       ;Ugrs, ha nem kell bekapcsolni.
    or      r11, #0x80          ;A tizedespont bekapcsolsa.
disp_wr_skip2:
    mov     (r9), r11           ;Megjelents a kijelzn.
    
    rts                         ;Visszatrs a hvhoz.


;*******************************************************************************
;* Az adatmemria inicializlsa a htszegmenses dekder tblzattal.          *
;*******************************************************************************
    DATA
    
bin_7seg:
    DB      0x3f, 0x06, 0x5b, 0x4f, 0x66, 0x6d, 0x7d, 0x07
    DB      0x7f, 0x6f, 0x77, 0x7c, 0x39, 0x5e, 0x79, 0x71
